@@ -19,6 +19,7 @@
  */
 #include <config.h>
 #include <gtk/gtk.h>
+#include <gdk/gdk.h>
 #include <glib.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -109,25 +110,25 @@ gint gtkb_load_pixmap(GtkbPixmap *pixmap, gchar *path, gchar *pixmapname) {
         if(!(fname=g_strconcat(path, pixmapname, NULL))) {
                 return 0;
         }
-  	if(pixmap->pixmap) gdk_pixmap_unref(pixmap->pixmap);
-  	if(pixmap->mask) gdk_bitmap_unref(pixmap->mask);
-        pixmap->pixmap=gdk_pixmap_create_from_xpm(MainWindow->window, &pixmap->mask,&MainStyle->bg[GTK_STATE_NORMAL], fname);
+  	if(pixmap->pixmap) g_object_unref(pixmap->pixmap);
+  	if(pixmap->mask) g_object_unref(pixmap->mask);
+        pixmap->pixmap=gdk_pixmap_create_from_xpm(MainWindow->window, &pixmap->mask, &MainStyle->bg[GTK_STATE_NORMAL], fname);
   	if(!pixmap->pixmap) {
         	perror(fname);
         	g_free(fname);
         	return 0;
   	}
         g_free(fname);
-  	gdk_window_get_size((GdkWindow *)pixmap->pixmap,&pixmap->xsize,&pixmap->ysize);
+  	gdk_drawable_get_size(GDK_DRAWABLE(pixmap->pixmap), &pixmap->xsize, &pixmap->ysize);
   	return 1;
 }
 
 void gtkb_pixmap_free(GtkbPixmap pixmap) {
 	if(pixmap.pixmap) {
-                gdk_pixmap_unref(pixmap.pixmap);
+                g_object_unref(pixmap.pixmap);
         }
 	if(pixmap.mask) {
-                gdk_bitmap_unref(pixmap.mask);
+                g_object_unref(pixmap.mask);
         }
 }
 
@@ -139,29 +140,31 @@ void gtkb_theme_free(GtkbTheme *theme) {
         }
 	gtkb_pixmap_free(theme->emptycell);
         for(i=0;i<8;i++) {
-        	gtkb_pixmap_free(theme->paws[i]);
+		gtkb_pixmap_free(theme->paws[i]);
         }
-        for(i=0;i<theme->numballs;i++) {
-        	gtkb_pixmap_free(theme->balls[i].ball);
-        	gtkb_pixmap_free(theme->balls[i].small);
-                if(theme->balls[i].jump) {
-                	for(j=0;j<theme->balls[i].jumpphases;j++) {
-                		gtkb_pixmap_free(theme->balls[i].jump[j]);
-                	}
-                	g_free(theme->balls[i].jump);
-                	if(theme->balls[i].jumpdelays) {
-				g_free(theme->balls[i].jumpdelays);
-                        }
-                }
-                if(theme->balls[i].destroy) {
-                	for(j=0;j<theme->balls[i].destroyphases;j++) {
-                		gtkb_pixmap_free(theme->balls[i].destroy[j]);
-                	}
-                	g_free(theme->balls[i].destroy);
-                	if(theme->balls[i].destroydelays) {
-                		g_free(theme->balls[i].destroydelays);
-                        }
-                }
+        if(theme->balls) {
+        	for(i=0;i<theme->numballs;i++) {
+        		gtkb_pixmap_free(theme->balls[i].ball);
+	        	gtkb_pixmap_free(theme->balls[i].small);
+        	        if(theme->balls[i].jump) {
+                		for(j=0;j<theme->balls[i].jumpphases;j++) {
+                			gtkb_pixmap_free(theme->balls[i].jump[j]);
+	                	}
+        	        	g_free(theme->balls[i].jump);
+                		if(theme->balls[i].jumpdelays) {
+					g_free(theme->balls[i].jumpdelays);
+	                        }
+        	        }
+                	if(theme->balls[i].destroy) {
+                		for(j=0;j<theme->balls[i].destroyphases;j++) {
+                			gtkb_pixmap_free(theme->balls[i].destroy[j]);
+	                	}
+        	        	g_free(theme->balls[i].destroy);
+                		if(theme->balls[i].destroydelays) {
+                			g_free(theme->balls[i].destroydelays);
+	                        }
+        	        }
+	        }
         }
         g_free(theme->balls);
         g_free(theme);
@@ -299,9 +302,9 @@ void remake_board(gint numoldchilds, gboolean nextwalid) {
         	SmallBalls=g_malloc(Rules.next*sizeof(GtkWidget *));
         	for(i=0;i<Rules.next;i++) {
                         if(nextwalid) {
-                		SmallBalls[i]=gtk_pixmap_new(gtkbTheme->balls[NextColors[i]-1].small.pixmap, gtkbTheme->balls[NextColors[i]-1].small.mask);
+                		SmallBalls[i]=gtk_image_new_from_pixmap(gtkbTheme->balls[NextColors[i]-1].small.pixmap, gtkbTheme->balls[NextColors[i]-1].small.mask);
                         } else {
-                		SmallBalls[i]=gtk_pixmap_new(gtkbTheme->balls[0].small.pixmap, gtkbTheme->balls[0].small.mask);
+                		SmallBalls[i]=gtk_image_new_from_pixmap(gtkbTheme->balls[0].small.pixmap, gtkbTheme->balls[0].small.mask);
                         }
   			gtk_box_pack_start(GTK_BOX(Small_balls_box), SmallBalls[i], FALSE, FALSE, 0);
         	}
@@ -310,9 +313,9 @@ void remake_board(gint numoldchilds, gboolean nextwalid) {
 
         cxs=gtkbTheme->emptycell.xsize;
         cys=gtkbTheme->emptycell.ysize;
-  	gtk_drawing_area_size(GTK_DRAWING_AREA(DrawingArea), Rules.xsize*cxs, Rules.ysize*cys);
+  	gtk_widget_set_size_request(DrawingArea, Rules.xsize*cxs, Rules.ysize*cys);
   	if(Pixmap) {
-		gdk_pixmap_unref(Pixmap);
+		g_object_unref(Pixmap);
   		Pixmap=gdk_pixmap_new(DrawingArea->window, Rules.xsize*cxs, Rules.ysize*cys, -1);
   	}
 }
@@ -338,7 +341,7 @@ void draw_next_balls(void) {
         gint i;
 
 	for(i=0;i<Rules.next;i++) {
-		gtk_pixmap_set(GTK_PIXMAP(SmallBalls[i]), gtkbTheme->balls[NextColors[i]-1].small.pixmap, gtkbTheme->balls[NextColors[i]-1].small.mask);
+		gtk_image_set_from_pixmap(GTK_IMAGE(SmallBalls[i]), gtkbTheme->balls[NextColors[i]-1].small.pixmap, gtkbTheme->balls[NextColors[i]-1].small.mask);
         }
 }
 
@@ -350,7 +353,7 @@ void draw_ball_no_update(GtkWidget *widget, gint ballcolor, GdkPixmap *pixmap, g
         gint  xr, yr;
 
 
-  	gdk_draw_pixmap(pixmap, gc, gtkbTheme->emptycell.pixmap, 0, 0, x*cxs, y*cys, cxs, cys);
+  	gdk_draw_drawable(pixmap, gc, gtkbTheme->emptycell.pixmap, 0, 0, x*cxs, y*cys, cxs, cys);
 	if(ballcolor) {
                 if(!jumpnum && !destroynum) {
                         ball=&gtkbTheme->balls[ballcolor-1].ball;
@@ -363,7 +366,7 @@ void draw_ball_no_update(GtkWidget *widget, gint ballcolor, GdkPixmap *pixmap, g
         	yr=y*cys+(cys-ball->ysize)/2;
         	gdk_gc_set_clip_mask(gc, ball->mask);
         	gdk_gc_set_clip_origin(gc, xr, yr);
-        	gdk_draw_pixmap(pixmap, gc, ball->pixmap, 0, 0, xr, yr, ball->xsize, ball->ysize);
+        	gdk_draw_drawable(pixmap, gc, ball->pixmap, 0, 0, xr, yr, ball->xsize, ball->ysize);
         	gdk_gc_set_clip_origin(gc, 0, 0);
         	gdk_gc_set_clip_mask(gc, 0);
   	}
@@ -385,10 +388,10 @@ void draw_paw(GtkWidget *widget, gint pawnumber, GdkPixmap *pixmap, gint x, gint
         gint  xr  = x*cxs+(cxs-paw->xsize)/2;
         gint  yr  = y*cys+(cys-paw->ysize)/2;
 
-  	gdk_draw_pixmap(pixmap, gc, gtkbTheme->emptycell.pixmap, 0, 0, x*cxs, y*cys, cxs, cys);
+  	gdk_draw_drawable(pixmap, gc, gtkbTheme->emptycell.pixmap, 0, 0, x*cxs, y*cys, cxs, cys);
   	gdk_gc_set_clip_mask(gc, paw->mask);
   	gdk_gc_set_clip_origin(gc, xr, yr);
-  	gdk_draw_pixmap(pixmap, gc, paw->pixmap, 0, 0, xr, yr, paw->xsize, paw->ysize);
+  	gdk_draw_drawable(pixmap, gc, paw->pixmap, 0, 0, xr, yr, paw->xsize, paw->ysize);
   	gdk_gc_set_clip_origin(gc, 0, 0);
   	gdk_gc_set_clip_mask(gc, 0);
   	update_rectangle(widget, x*cxs, y*cys, cxs, cys);
@@ -409,7 +412,7 @@ void draw_board(GtkWidget *widget, GdkPixmap *pixmap) {
 
 /* Refill the screen from the backing pixmap */
 gint expose_event(GtkWidget *widget, GdkEventExpose *event) {
-  	gdk_draw_pixmap(widget->window, widget->style->fg_gc[GTK_WIDGET_STATE(widget)], Pixmap,
+  	gdk_draw_drawable(widget->window, widget->style->fg_gc[GTK_WIDGET_STATE(widget)], Pixmap,
 		   	event->area.x, event->area.y, event->area.x, event->area.y,
 			event->area.width, event->area.height);
   	return FALSE;
