@@ -17,17 +17,21 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
+#include <errno.h>
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
+#include <signal.h>
+#include <unistd.h>
+#include <fcntl.h>
 #include <locale.h>
 #include <sys/param.h>
 #include <time.h>
 #include <gtk/gtk.h>
-#include <gdk/gdkkeysyms.h>
-#include <X11/Xlocale.h>
+#include <gdk-pixbuf/gdk-pixbuf.h>
 
 #include "gtkballs.h"
+#include "gtkutils.h"
 #include "interface.h"
 #include "preferences.h"
 #include "scoreboard.h"
@@ -66,6 +70,11 @@ GtkWidget *DrawingArea=NULL;
 /* Score labels (also used in interface.[ch]) */
 GtkWidget *Hi_score_label=NULL,*User_score_label=NULL;
 
+GtkWidget *Timer_label=NULL;
+
+gint Timer_limit=-1;
+time_t Timer_start_time=-1;
+
 /* box for next balls */
 GtkWidget *Small_balls_box;
 
@@ -73,6 +82,8 @@ struct score_board Scoreboard[10];
 struct score_board_full *FullScoreboard=NULL;
 gint FullScoreboardNumber=0;
 
+int Score_fds[2];
+
 /* This flag shows should be next colors to be shown
  * This variable is also used in preferences.[ch] and interface.c */
 gint Show_next_colors=TRUE;
@@ -121,32 +132,32 @@ GtkItemFactory *Menu_items_factory;
 
 static GtkItemFactoryEntry Menu_items[] = 
 {
-	{N_("/Game"), 		      NULL, 	    NULL, 		0, "<Branch>"},
-  	{N_("/Game/tear0"), 	      NULL, 	    NULL, 		0, "<Tearoff>"},
-  	{N_("/Game/New"), 	      "<control>N", new_game, 		0, "<Item>"},
-  	{N_("/Game/Rules"), 	      "<control>R", rules_dialog, 	0, "<Item>"},
-  	{N_("/Game/sep0"), 	      NULL, 	    NULL, 		0, "<Separator>"},
-  	{N_("/Game/Save"), 	      "<control>S", save_game_cb, 	0, "<Item>"},
-  	{N_("/Game/Load"), 	      "<control>L", load_game_cb,      	0, "<Item>"},
-  	{N_("/Game/sep0"), 	      NULL, 	    NULL, 		0, "<Separator>"},
-        {N_("/Game/Hall of fame"),    "<control>H", show_hall_of_fame,  0, "<Item>"},
-        {N_("/Game/sep1"), 	      NULL, 	    NULL, 		0, "<Separator>"},
-  	{N_("/Game/Quit"), 	      "<control>Q", destroy, 		0, "<Item>"},
-  	{N_("/Edit"), 	      	      NULL, 	    NULL, 		0, "<Branch>"},
-  	{N_("/Edit/tear1"),           NULL,         NULL, 		0, "<Tearoff>"},
-  	{N_("/Edit/Undo"), 	      "<control>U", undo_move, 		0, "<Item>"},
-  	{N_("/Settings"), 	      NULL, 	    NULL, 		0, "<Branch>"},
-  	{N_("/Settings/tear1"),       NULL,         NULL, 		0, "<Tearoff>"},
-  	{N_("/Settings/Preferences"), "<control>P", preferences_dialog, 0, "<Item>"},
-  	{N_("/Help"),                 NULL,         NULL, 		0, "<LastBranch>"},
-  	{N_("/Help/tear2"),           NULL,         NULL, 		0, "<Tearoff>"},
-  	{N_("/Help/Rules"), 	      "F1",         show_rules, 	0, "<Item>"},
-  	{N_("/Help/sep2"),  	      NULL,         NULL, 		0, "<Separator>"},
-  	{N_("/Help/About"), 	      "<control>A", about, 		0, "<Item>"}
+	{N_("/_Game"), 		       NULL, 	     NULL, 		0, "<Branch>"},
+  	{N_("/_Game/tear0"), 	       NULL, 	     NULL, 		0, "<Tearoff>"},
+  	{N_("/_Game/_New"), 	       "<control>N", new_game, 		0, "<StockItem>", GTK_STOCK_NEW},
+  	{N_("/_Game/_Rules"), 	       "<control>R", rules_dialog, 	0, "<StockItem>", GTK_STOCK_PROPERTIES},
+  	{N_("/_Game/sep0"), 	       NULL, 	     NULL, 		0, "<Separator>"},
+  	{N_("/_Game/_Save"), 	       "<control>S", save_game_cb, 	0, "<StockItem>", GTK_STOCK_SAVE},
+  	{N_("/_Game/_Load"), 	       "<control>L", load_game_cb,      0, "<StockItem>", GTK_STOCK_OPEN},
+  	{N_("/_Game/sep0"), 	       NULL, 	     NULL, 		0, "<Separator>"},
+        {N_("/_Game/_Hall of fame"),   "<control>H", show_hall_of_fame, 0, "<Item>"},
+        {N_("/_Game/sep1"), 	       NULL, 	     NULL, 		0, "<Separator>"},
+  	{N_("/_Game/_Quit"), 	       "<control>Q", gtk_main_quit,     0, "<StockItem>", GTK_STOCK_QUIT},
+  	{N_("/_Edit"), 	      	       NULL, 	     NULL, 		0, "<Branch>"},
+  	{N_("/_Edit/tear1"),           NULL,         NULL, 		0, "<Tearoff>"},
+  	{N_("/_Edit/_Undo"), 	       "<control>U", undo_move,        	0, "<StockItem>", GTK_STOCK_UNDO},
+  	{N_("/_Settings"), 	       NULL, 	     NULL, 		0, "<Branch>"},
+  	{N_("/_Settings/tear1"),       NULL,         NULL, 		0, "<Tearoff>"},
+  	{N_("/_Settings/_Preferences"),"<control>P", preferences_dialog,0, "<StockItem>", GTK_STOCK_PREFERENCES},
+  	{N_("/_Help"),                 NULL,         NULL, 		0, "<LastBranch>"},
+  	{N_("/_Help/tear2"),           NULL,         NULL, 		0, "<Tearoff>"},
+  	{N_("/_Help/_Rules"), 	       "F1",         show_rules, 	0, "<StockItem>", GTK_STOCK_HELP},
+  	{N_("/_Help/sep2"),  	       NULL,         NULL, 		0, "<Separator>"},
+  	{N_("/_Help/_About"),	       "<control>A", about, 		0, "<Item>"}
 };
 
 static gchar *menu_translate(const gchar *path, gpointer data) {
-        gchar *retval=gettext(path);
+	gchar *retval=gettext(path);
 
         return retval;
 }
@@ -174,12 +185,6 @@ void get_main_menu(GtkWidget *window, GtkWidget **menubar) {
         }
 }
 
-void key_pressed(GtkWidget *widget, GdkEventKey *event) {
-	if(event && event->keyval == GDK_Escape) {
-                gtk_widget_destroy(widget);
-        }
-}
-
 void undo_move(GtkWidget *widget, gpointer data) {
 
         memcpy(Board, BoardUndo, sizeof(gint)*Rules.xsize*Rules.ysize);
@@ -195,7 +200,7 @@ void undo_move(GtkWidget *widget, gpointer data) {
         menu_set_sensitive(Menu_items_factory, "/Edit/Undo", FALSE);
         draw_board(DrawingArea, Pixmap);
         draw_next_balls();
-        gtk_widget_draw(DrawingArea, NULL);
+        update_rectangle(DrawingArea, 0, 0, -1, -1);
 }
 
 gint count_free_cells(void) {
@@ -209,36 +214,51 @@ gint count_free_cells(void) {
   	return counter;
 }
 
-gint new_turn(gint number, gboolean first) {
+void new_turn(gint number, gboolean first) {
   	gint i,k=0,free_cells_number;
 
-  	if(!(free_cells_number=count_free_cells())) {
-    		return FALSE;
-        }
+        do {
+        	Timer_start_time=time(NULL);
 
-  	k=0;
-        if(number < Rules.next) {
-                number = Rules.next;
-        }
-  	do {
-      		i=(gint)(((gfloat)Rules.xsize*Rules.ysize)*rand()/(RAND_MAX+1.0));
-      		if(Board[i] == 0) {
-                        if(first) {
-                                Board[i]=1+(gint)(((gfloat)Rules.colors)*rand()/(RAND_MAX+1.0));
-                        } else {
-	  			Board[i]=NextColors[k];
-                        }
-	  		k++;
-		} else {
-			continue;
-                }
-      		/* I hope that k!=0 */
-                if(k<=Rules.next) {
-      			NextColors[k-1]=1+(gint)(((gfloat)Rules.colors)*rand()/(RAND_MAX+1.0));
-                }
-    	} while(k<((number<free_cells_number) ? number : free_cells_number));
+	  	k=0;
+        	if(number < Rules.next) {
+                	number = Rules.next;
+	        }
 
-  	return TRUE;
+  		if((free_cells_number=count_free_cells()) <= number) {
+			if (Score>Scoreboard[9].score) {
+				input_name_dialog();
+	                }
+		      	new_game();
+    			return;
+	        }
+
+  		do {
+      			i=(gint)(((gfloat)Rules.xsize*Rules.ysize)*rand()/(RAND_MAX+1.0));
+	      		if(Board[i] == 0) {
+        	                if(first) {
+                	                Board[i]=1+(gint)(((gfloat)Rules.colors)*rand()/(RAND_MAX+1.0));
+                        	} else {
+	  				Board[i]=NextColors[k];
+	                        }
+		  		k++;
+			} else {
+				continue;
+	                }
+      			/* I hope that k!=0 */
+                	if(k<=Rules.next) {
+      				NextColors[k-1]=1+(gint)(((gfloat)Rules.colors)*rand()/(RAND_MAX+1.0));
+	                }
+    		} while(k<((number<free_cells_number) ? number : free_cells_number));
+
+                if(!first) {
+	        	draw_next_balls();
+			/* if line complete, don't increase user's score--it's not his merit */
+			destroy_lines(DrawingArea, Pixmap, FALSE);
+	  		draw_board(DrawingArea, Pixmap);
+	        	update_rectangle(DrawingArea, 0, 0, -1, -1);
+                }
+        } while((free_cells_number=count_free_cells()) == Rules.xsize*Rules.ysize);
 }
 
 /* 
@@ -262,34 +282,29 @@ void new_game(void){
   	if(DrawingArea != NULL) {
                 draw_next_balls();
         	draw_board(DrawingArea, Pixmap);
-      		gtk_widget_draw(DrawingArea, NULL);
+        	update_rectangle(DrawingArea, 0, 0, -1, -1);
     	}
 }
 
 /* Show dialog box with game rules*/
 void show_rules(GtkWidget *widget, gpointer data) {
   	GtkWidget *window;
-  	GtkWidget *vbox,*hbox,*button_box;
+  	GtkWidget *vbox, *hbox, *button_box;
   	GtkWidget *label;
+  	GtkWidget *frame;
   	GtkWidget *separator;
   	GtkWidget *ok_button;
 
-  	window=gtk_widget_new(gtk_window_get_type(),
-			      "GtkObject::user_data", NULL,
-			      "GtkWindow::type", GTK_WINDOW_TOPLEVEL,
-			      "GtkWindow::allow_grow", FALSE,
-			      "GtkWindow::allow_shrink", FALSE, NULL);
-  	gtk_window_set_title(GTK_WINDOW(window), _("Rules"));
-        gtk_window_set_wmclass(GTK_WINDOW(window), "GtkBalls_Rules", "GtkBalls");
-  	gtk_signal_connect(GTK_OBJECT(window), "delete_event", GTK_SIGNAL_FUNC(gtk_widget_destroy), NULL);
-  	gtk_signal_connect(GTK_OBJECT(window), "key_press_event", GTK_SIGNAL_FUNC(key_pressed), NULL);
-  	gtk_container_border_width(GTK_CONTAINER(window), 2);
-  
+	window=ut_window_new(_("Rules"), "GtkBalls_Rules", "GtkBalls", TRUE, FALSE, FALSE, 5);
+
   	vbox=gtk_vbox_new(FALSE, 0);
   	gtk_container_add(GTK_CONTAINER(window), vbox);
 
+        frame=gtk_frame_new(_("Rules"));
+  	gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 3);
+
   	hbox=gtk_hbox_new(FALSE, 0);
-  	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
+	gtk_container_add(GTK_CONTAINER(frame), hbox);
 
   	label=gtk_label_new(_("The play area of GtkBalls is a 9x9 grid. When GtkBalls  is\n" \
 			      "first  started  a  number  of balls are placed in a random\n" \
@@ -311,7 +326,7 @@ void show_rules(GtkWidget *widget, gpointer data) {
 			      "decide which ball you are going to move, there is an indi-\n" \
 			      "cator on top of the grid that shows what colors  the  next\n" \
 			      "balls that will appear on the grid will be."));
-  	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 2);
+  	gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 5);
 
   	separator=gtk_hseparator_new();
   	gtk_box_pack_start(GTK_BOX(vbox), separator, FALSE, FALSE, 5);
@@ -319,13 +334,10 @@ void show_rules(GtkWidget *widget, gpointer data) {
   	button_box=gtk_hbox_new(FALSE, 0);
   	gtk_box_pack_start(GTK_BOX(vbox), button_box, TRUE, TRUE, 2);
 
-  	ok_button=gtk_button_new_with_label(_("OK"));
-        GTK_WIDGET_SET_FLAGS(ok_button, GTK_CAN_DEFAULT | GTK_CAN_FOCUS);
-  	gtk_signal_connect_object(GTK_OBJECT(ok_button), "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(window));
-  	gtk_box_pack_start(GTK_BOX(button_box), ok_button, TRUE, TRUE, 0);
+        ok_button=ut_button_new_stock_swap(GTK_STOCK_CLOSE, gtk_widget_destroy, window, button_box);
 
-  	gtk_widget_show_all(window);
         gtk_widget_grab_default(ok_button);
+  	gtk_widget_show_all(window);
 }
 
 /* Show about dialog box */
@@ -340,19 +352,9 @@ void about(GtkWidget *widget, gpointer data) {
   	GtkWidget *separator;
   	GtkWidget *ok_button, *license_button;
   	gchar     strbuf[1024];
-  
-  	window=gtk_widget_new(gtk_window_get_type(),
-			      "GtkObject::user_data", NULL,
-			      "GtkWindow::type", GTK_WINDOW_TOPLEVEL,
-			      "GtkWindow::allow_grow", FALSE,
-			      "GtkWindow::modal", TRUE,
-			      "GtkWindow::allow_shrink", FALSE, NULL);
-  	gtk_object_set_user_data(GTK_OBJECT(window), "about");
-  	gtk_window_set_title(GTK_WINDOW (window), _("About"));
-        gtk_window_set_wmclass(GTK_WINDOW(window), "GtkBalls_About", "GtkBalls");
-  	gtk_signal_connect(GTK_OBJECT(window), "key_press_event", GTK_SIGNAL_FUNC(key_pressed), NULL);
-  	gtk_container_border_width(GTK_CONTAINER(window), 2);
-  
+
+	window=ut_window_new(_("About"), "GtkBalls_About", "GtkBalls", TRUE, TRUE, FALSE, 5);
+
   	vbox=gtk_vbox_new(FALSE, 0);
   	gtk_container_add(GTK_CONTAINER(window), vbox);
 
@@ -364,12 +366,12 @@ void about(GtkWidget *widget, gpointer data) {
 				          &style->bg[GTK_STATE_NORMAL],
 				          DATADIR "/gtkballs/gtkballs-logo.xpm");
   	if(splash) {
-		pixmap=gtk_pixmap_new(splash, mask);
+        	pixmap=gtk_image_new_from_pixmap(splash, mask);
   		gtk_box_pack_start(GTK_BOX(hbox), pixmap, TRUE, TRUE, 4);
         }
 
   	g_snprintf(strbuf, sizeof(strbuf), _("GtkBalls %s\n Copyright (C) 1998-1999 Eugene Morozov\n" \
-					     "<jmv@lucifer.dorms.spbu.ru>, <john_morozov@yahoo.com>\n\n" \
+					     "<jmv@online.ru>, <roshiajin@yahoo.com>\n\n" \
 		       		       	     "GtkBalls comes with ABSOLUTELY NO WARRANTY;\n" \
 		       		       	     "for details press \"Show license\" button.\n" \
       		       		       	     "This is free software and you are welcome to\n" \
@@ -385,23 +387,12 @@ void about(GtkWidget *widget, gpointer data) {
   	gtk_box_pack_start(GTK_BOX(vbox), buttons_box, TRUE, TRUE, 2);
         gtk_button_box_set_layout(GTK_BUTTON_BOX(buttons_box), GTK_BUTTONBOX_SPREAD);
 
+        ok_button=ut_button_new_stock_swap(GTK_STOCK_CLOSE, gtk_widget_destroy, window, buttons_box);
 
-  	ok_button=gtk_button_new_with_label("OK");
-        GTK_WIDGET_SET_FLAGS(ok_button, GTK_CAN_DEFAULT | GTK_CAN_FOCUS);
-  	gtk_signal_connect(GTK_OBJECT(ok_button), "clicked", GTK_SIGNAL_FUNC(destroy), GTK_OBJECT(window));
-  	gtk_box_pack_start(GTK_BOX(buttons_box), ok_button, TRUE, TRUE, 0);
+        license_button=ut_button_new(_("Show license"), gnu_license_dialog, window, buttons_box);
 
-  	license_button=gtk_button_new_with_label(_(" Show license "));
-        GTK_WIDGET_SET_FLAGS(license_button, GTK_CAN_DEFAULT | GTK_CAN_FOCUS);
-  	gtk_signal_connect(GTK_OBJECT(license_button), "clicked", GTK_SIGNAL_FUNC(gnu_license_dialog), window);
-  	gtk_box_pack_start(GTK_BOX(buttons_box), license_button, TRUE, TRUE, 0);
-
-  	gtk_widget_show_all(window);
         gtk_widget_grab_default(ok_button);
-}
-
-gint delete_event(GtkWidget *widget, gpointer data) {
-  	return (FALSE);
+  	gtk_widget_show_all(window);
 }
 
 gint animate_ball(gpointer data) {
@@ -424,9 +415,29 @@ gint animate_ball(gpointer data) {
   	return FALSE;
 }
 
+gint countdown_timer(gpointer data) {
+        gchar *text;
+        GtkLabel *label = data;
+        time_t nowt;
+        gint trem;
+
+        if(Timer_start_time == -1 || Timer_limit <= 0) return TRUE;
+
+        nowt=time(NULL);
+        if(nowt - Timer_start_time >= Timer_limit) {
+        	Timer_start_time=nowt;
+	      	new_turn(Rules.next, FALSE);
+        }
+        trem=Timer_limit + Timer_start_time - nowt;
+        text=g_strdup_printf(_("Remaining time: %02d:%02d"), trem/60, trem%60);
+        gtk_label_set_text(label, text);
+        g_free(text);
+
+  	return TRUE;
+}
+
 gint button_press_event(GtkWidget *widget, GdkEventButton *event) {
   	gint x,y,xn,yn;
-        gint fcc; /* number of free cells left */
 	gint *TmpBoardUndo;
 	gint *TmpNextColorsUndo;
 	gint TmpScoreUndo, TmpHi_scoreUndo;
@@ -472,63 +483,112 @@ gint button_press_event(GtkWidget *widget, GdkEventButton *event) {
                         ScoreUndo=TmpScoreUndo;
                         Hi_scoreUndo=TmpHi_scoreUndo;
         		menu_set_sensitive(Menu_items_factory, "/Edit/Undo", TRUE);
+
 	  		if(!destroy_lines(widget, Pixmap, TRUE)) {
 	      			new_turn(Rules.next, FALSE);
-                		draw_next_balls();
 	    		}
-	  		/* if fifth ball will appear, don't increase user's
-	     		   score--it's not his merit */
-	  		destroy_lines(widget, Pixmap, FALSE);
-	  		/* if there's no free cells start a new game */
-	  		if(!(fcc=count_free_cells())) {
-	      			if (Score>Scoreboard[9].score) {
-					input_name_dialog();
-                                }
-	      			new_game();
-	    		} else if(fcc==Rules.xsize*Rules.ysize) { /* no balls left on the board */
-	      			new_turn(Rules.next, FALSE);
-                		draw_next_balls();
-                        }
 
-	  		draw_board(widget, Pixmap);
-	  		gtk_widget_draw(widget, NULL);
+                        Timer_start_time=time(NULL);
 		}
     	}
 
   	return FALSE;
 }
 
-void destroy(GtkWidget *widget, gpointer data) {
-  	if(data) {
-      		gtk_widget_destroy(GTK_WIDGET(data));
-      		return;
-  	}
-  	gtk_main_quit();
+void child_process_score_writer(int ch_fds[2]) {
+	fd_set rfds;
+        int i;
+  	int fd=-1;
+	gchar *buf;
+        size_t sz;
+  	sigset_t sset;
+	struct flock lockinfo;
+
+        close(ch_fds[1]);
+        while(1) {
+        	FD_ZERO(&rfds);
+                FD_SET(ch_fds[0], &rfds);
+                i=select(ch_fds[0] + 1, &rfds, NULL, NULL, NULL);
+                if(i > 0) {
+                	i = read(ch_fds[0], &sz, sizeof(sz));
+                        if(i <= 0) {
+                        	exit(0);
+                        }
+                        /* block signals before writing, to prevent possible file corruption */
+			sigemptyset(&sset);
+			sigaddset(&sset, SIGHUP);
+		  	sigaddset(&sset, SIGINT);
+			sigaddset(&sset, SIGQUIT);
+		  	sigaddset(&sset, SIGTERM);
+			sigprocmask(SIG_BLOCK, &sset, NULL);
+  			fd=open(LOCALSTATEDIR SCORE_FILE, O_WRONLY | O_TRUNC);
+                        if(fd != -1) {
+  				/* get write lock before writing scores */
+  				lockinfo.l_whence=SEEK_SET;
+  				lockinfo.l_start=0;
+	  			lockinfo.l_len=0;
+
+  				i=0;
+  				while(1) {
+    					lockinfo.l_type=F_WRLCK;
+    					if(!fcntl(fd, F_SETLK, &lockinfo)) break;
+	    				if(i>=3) {
+      						close(fd);
+      						fd=-1;
+    					}
+    					i++;
+	  			}
+                        }
+                        while(sz > 0) {
+                                buf=g_malloc(sz);
+                		read(ch_fds[0], buf, sz);
+                        	if(fd != -1) {
+                                        write(fd, buf, sz);
+                                }
+                                g_free(buf);
+                		read(ch_fds[0], &sz, sizeof(sz));
+                        }
+
+                        if(fd != -1) {
+  				close(fd);
+                        }
+  			sigprocmask(SIG_UNBLOCK, &sset, NULL);
+                }
+        }
 }
 
 int main(int argc, char **argv) {
   	GtkWidget *menubar;
-  	GtkWidget *vbox,*hbox,*drawing_area_box;
-  	GdkBitmap *mask;
-  	GdkPixmap *icon;
+  	GtkWidget *vbox,*hbox,*hbox1,*drawing_area_box;
+  	GdkPixbuf *icon;
   	gint i;
+	GError *error=NULL;
+        gchar *err;
+
+        pid_t pid;
 
   	gtk_set_locale();
 #ifdef HAVE_NLS
   	bindtextdomain (PACKAGE, LOCALEDIR);
   	textdomain (PACKAGE);
+        bind_textdomain_codeset(PACKAGE, "UTF8");
 #endif /* HAVE_NLS */
 
-#if 0
-        if(!sb_init_score()) {
-                printf("scores will NOT be saved.\n");
+        if(pipe(Score_fds) == -1) {
+                printf("pipe() failed: %s\n", strerror(errno));
         }
-        /* drop priveleges here */
-#endif
-
-  	if(!read_score(Scoreboard, &FullScoreboard, &FullScoreboardNumber)) {
-                printf(_("Unable to read score.\n"));
-    	}
+        pid = fork();
+        if(pid == -1) {
+                printf("cannot fork: %s\n", strerror(errno));
+        } else if(pid == 0) {
+                child_process_score_writer(Score_fds);
+        }
+/*
+        printf("%d\n", getgid());
+        setgid(getgid());
+*/
+        /* drop privileges */
+        setregid(getgid(), getgid());
 
   	/* load user's preferences */
   	load_preferences();
@@ -540,25 +600,24 @@ int main(int argc, char **argv) {
         NextColors=g_malloc(sizeof(gint)*Rules.next);
         NextColorsUndo=g_malloc(sizeof(gint)*Rules.next);
   	new_turn(Rules.destroy, TRUE);
+        Timer_start_time=-1;
 
   	gtk_init(&argc, &argv);
 
-  	MainWindow=gtk_widget_new(gtk_window_get_type(),
-				   "GtkObject::user_data", NULL,
-				   "GtkWindow::type", GTK_WINDOW_TOPLEVEL,
-				   "GtkWindow::title", "GtkBalls",
-				   "GtkWindow::allow_grow", FALSE,
-				   "GtkWindow::allow_shrink", FALSE, NULL);
-        gtk_window_set_wmclass(GTK_WINDOW(MainWindow), "GtkBalls_Main", "GtkBalls");
+  	if(!read_score(Scoreboard, &FullScoreboard, &FullScoreboardNumber)) {
+                simple_message_box(_("Unable to read score.\n"));
+    	}
+
+	MainWindow=ut_window_new(_("GtkBalls"), "GtkBalls_Main", "GtkBalls", FALSE, FALSE, FALSE, 0);
   	MainStyle=gtk_widget_get_style(MainWindow);
 
-  	gtk_signal_connect(GTK_OBJECT(MainWindow), "delete_event", GTK_SIGNAL_FUNC(delete_event), NULL);
-  	gtk_signal_connect(GTK_OBJECT(MainWindow), "destroy", GTK_SIGNAL_FUNC (destroy), NULL);
+  	g_signal_connect(G_OBJECT(MainWindow), "destroy", G_CALLBACK(gtk_main_quit), MainWindow);
   	gtk_widget_realize(MainWindow);
   	DrawingArea=gtk_drawing_area_new();
 
   	vbox=gtk_vbox_new(FALSE, 5);
-  	hbox=gtk_hbox_new(TRUE, 0);
+  	hbox=gtk_hbox_new(FALSE, 0);
+  	hbox1=gtk_hbox_new(FALSE, 0);
   	drawing_area_box=gtk_hbox_new(FALSE, 0);
   	Small_balls_box=gtk_hbox_new(TRUE, 0);
 
@@ -571,37 +630,45 @@ int main(int argc, char **argv) {
 
   	Hi_score_label=gtk_label_new(_("Hi-score: 0"));
   	set_hi_score_label();
-  	gtk_box_pack_start(GTK_BOX(hbox), Hi_score_label, FALSE, FALSE, 0);
+  	gtk_box_pack_start(GTK_BOX(hbox), Hi_score_label, FALSE, FALSE, 5);
 
   	User_score_label=gtk_label_new(_("Your score: 0"));
   	set_user_score_label();
-  	gtk_box_pack_end(GTK_BOX(hbox), User_score_label, FALSE, FALSE, 0);
+  	gtk_box_pack_end(GTK_BOX(hbox), User_score_label, FALSE, FALSE, 5);
+
+        Timer_label=gtk_label_new(NULL);
+  	gtk_box_pack_start(GTK_BOX(hbox1), Timer_label, TRUE, TRUE, 5);
+	gtk_timeout_add(250, countdown_timer, Timer_label);
 
   	gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(hbox), FALSE, FALSE, 0);
+  	gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(hbox1), FALSE, FALSE, 0);
 
-  	gtk_signal_connect(GTK_OBJECT(DrawingArea), "expose_event", (GtkSignalFunc)expose_event, NULL);
-  	gtk_signal_connect(GTK_OBJECT(DrawingArea), "button_press_event", (GtkSignalFunc)button_press_event, NULL);
+  	g_signal_connect(G_OBJECT(DrawingArea), "expose_event", G_CALLBACK(expose_event), NULL);
+  	g_signal_connect(G_OBJECT(DrawingArea), "button_press_event", G_CALLBACK(button_press_event), NULL);
   	gtk_widget_set_events(DrawingArea, GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK);
 
   	gtk_box_pack_start(GTK_BOX(drawing_area_box), DrawingArea, TRUE, FALSE, 10);
   	gtk_box_pack_start(GTK_BOX(vbox), drawing_area_box, FALSE, FALSE, 10);
   	gtk_container_add(GTK_CONTAINER(MainWindow), vbox);
 
-  	icon=gdk_pixmap_create_from_xpm(MainWindow->window, &mask,
-				        &MainStyle->bg[GTK_STATE_NORMAL],
-				        DATADIR "/gtkballs/gtkballs_16x16.xpm");
+        icon=gdk_pixbuf_new_from_file(DATADIR "/gtkballs/gtkballs_16x16.xpm", &error);
+        if(error) g_error_free(error);
   	if(icon) {
-		gdk_window_set_icon(MainWindow->window, NULL, icon, mask);
+		gtk_window_set_icon(GTK_WINDOW(MainWindow), icon);
         }
 
   	if(!(i=load_theme(Theme_name))) {
                 if(strcmp(Theme_name, Default_theme_name)!=0) {
-                	g_warning(_("Failed loading theme \"%s\"! Trying \"%s\"\n"),Theme_name, Default_theme_name);
+                        err=g_strdup_printf(_("Failed loading theme \"%s\"! Trying \"%s\"\n"),Theme_name, Default_theme_name);
+                	simple_message_box(err);
+                        g_free(err);
                 	Theme_name=Default_theme_name;
   			i=load_theme(Theme_name);
                 }
                 if(!i) {
-        		g_warning(_("Failed loading theme \"%s\"! Exiting.\n"),Theme_name);
+                        err=g_strdup_printf(_("Failed loading theme \"%s\"! Exiting.\n"),Theme_name);
+                	simple_message_box(err);
+                        g_free(err);
                         return 1;
                 }
   	}
@@ -611,7 +678,7 @@ int main(int argc, char **argv) {
 
         SmallBalls=g_malloc(Rules.next*sizeof(GtkWidget *));
         for(i=0;i<Rules.next;i++) {
-                SmallBalls[i]=gtk_pixmap_new(gtkbTheme->balls[NextColors[i]-1].small.pixmap, gtkbTheme->balls[NextColors[i]-1].small.mask);
+                SmallBalls[i]=gtk_image_new_from_pixmap(gtkbTheme->balls[NextColors[i]-1].small.pixmap, gtkbTheme->balls[NextColors[i]-1].small.mask);
   		gtk_box_pack_start(GTK_BOX(Small_balls_box), SmallBalls[i], FALSE, FALSE, 0);
         }
 
@@ -622,6 +689,10 @@ int main(int argc, char **argv) {
   	gtk_widget_show(Small_balls_box);
         draw_board(DrawingArea, Pixmap);
         gtk_quit_add(0, (GtkFunction)gtkb_theme_free_handler, NULL);
+
+        Timer_start_time=time(NULL);
+        gtk_label_set_text(GTK_LABEL(Timer_label), _("Time is unlimited"));
+
   	gtk_main();
 
   	return 0;
