@@ -42,7 +42,7 @@
 static GtkWidget *dialog 	= NULL;
 static gint 	  saved_score 	= 0;
 static gchar 	 *user_name 	= NULL;
-static gchar 	  last_player_name[15];
+static gchar 	  last_player_name[15]="";
 
 gchar *selected_save_load=NULL;
 
@@ -188,13 +188,7 @@ gint get_saved_games(gchar ***gamelist) {
         return num;
 }
 
-void save_load_game_row_selected(GtkWidget *clist, gint row) {
-        selected_save_load=gtk_clist_get_row_data(GTK_CLIST(clist), row);
-}
-
 void do_save_game(GtkWidget *widget, gpointer data) {
-/* maybe i should do (in feature) some endianess conversion here... */
-/* and handle different int sizes on different architectures? =) */
         FILE *f;
         gchar *fname, *errormsg;
         time_t nowtime;
@@ -222,7 +216,7 @@ void do_save_game(GtkWidget *widget, gpointer data) {
                 fclose(f);
         } else {
                 errormsg=g_strdup_printf(_("Cannot save game to:\n%s\n%s"), fname, strerror(errno));
-                simple_message_box(errormsg, _("OK"));
+                simple_message_box(errormsg);
                 g_free(errormsg);
         }
 
@@ -254,7 +248,7 @@ void reinit_board(gint *newboard, gint *newnext, gint score, gint oldnext) {
         }
 
   	if(!read_score(Scoreboard, &FullScoreboard, &FullScoreboardNumber)) {
-      		simple_message_box(_("Unable to read score.\n"), _("OK"));
+      		simple_message_box(_("Unable to read score.\n"));
     	}
 
         Score=score;
@@ -269,7 +263,7 @@ void reinit_board(gint *newboard, gint *newnext, gint score, gint oldnext) {
   		if(DrawingArea != NULL) {
                 	draw_next_balls();
         		draw_board(DrawingArea, Pixmap);
-      			gtk_widget_draw(DrawingArea, NULL);
+      			update_rectangle(DrawingArea, 0, 0, -1, -1);
     		}
         }
 }
@@ -280,7 +274,7 @@ void do_load_game(GtkWidget *widget, gpointer data) {
         GtkbGameRules rules;
 
         if(!selected_save_load) {
-                simple_message_box(_("No game selected for load.\n"), _("OK"));
+                simple_message_box(_("No game selected for load.\n"));
   		if(data) {
       			gtk_widget_destroy(GTK_WIDGET(data));
 	  	}
@@ -289,14 +283,14 @@ void do_load_game(GtkWidget *widget, gpointer data) {
 
 	if(!parse_save_game(selected_save_load, &rules, &score, &board, &next)) {
         	errormsg=g_strdup_printf(_("Cannot load game from:\n%s\n"), selected_save_load);
-        	simple_message_box(errormsg, _("OK"));
+        	simple_message_box(errormsg);
                 g_free(errormsg);
 		return;
         }
 
         if(rules.colors > gtkbTheme->numballs) {
         	errormsg=g_strdup_printf(_("Not enough balls(%d) in current theme.\nWe need %d balls.\nLoad another theme and try again."), gtkbTheme->numballs, rules.colors);
-        	simple_message_box(errormsg, _("OK"));
+        	simple_message_box(errormsg);
                 g_free(errormsg);
 		return;
         }
@@ -315,103 +309,124 @@ void do_load_game(GtkWidget *widget, gpointer data) {
   	}
 }
 
+void save_row_activated_cb(GtkTreeView *treeview, gpointer arg1, GtkTreeViewColumn *arg2, gpointer data) {
+        do_save_game(GTK_WIDGET(treeview), data);
+}
+
+void load_row_activated_cb(GtkTreeView *treeview, gpointer arg1, GtkTreeViewColumn *arg2, gpointer data) {
+        do_load_game(GTK_WIDGET(treeview), data);
+}
+
 void save_load_game_list_destroy(gpointer data) {
         if(data) g_free(data);
 }
 
+void sl_row_activated(GtkTreeSelection *selection, GtkTreeModel *model) {
+        GtkTreeIter iter;
+        GValue value = {0, };
+
+        if(gtk_tree_selection_get_selected(selection, NULL, &iter)) {
+                gtk_tree_model_get_value(model, &iter, 1, &value);
+                if(g_value_get_string(&value)) {
+                        selected_save_load=(gchar *)g_value_get_string(&value);
+                } else {
+                        selected_save_load=NULL;
+                }
+        }
+}
+
 void save_load_game_dialog(GtkWidget *widget, gpointer data, gboolean is_save) {
+        GtkListStore *store;
+        GtkTreeIter iter;
+	GtkWidget *treeview;
+        GtkCellRenderer *renderer;
+        GtkTreeViewColumn *column;
+        GtkTreePath *path;
   	GtkWidget *window, *swindow;
   	GtkWidget *vbox, *button_box;
   	GtkWidget *ok_button, *cancel_button;
-  	GtkWidget *clist;
-        GtkStyle *style;
-
-        gint      i, num, width=0, w;
-        gchar     *titles=_("Date (score)"), *empty=_("Empty");
+        gint      i, num;
         gchar     **gamelist;
 
         num=get_saved_games(&gamelist);
         if(!is_save && !num) {
-                simple_message_box(_("No saved games found.\n"), _("OK"));
+                simple_message_box(_("No saved games found.\n"));
                 return;
         }
-  	window=gtk_widget_new(gtk_window_get_type(),
-		       	      "GtkObject::user_data", NULL,
-			      "GtkWindow::allow_grow", TRUE,
-			      "GtkWindow::allow_shrink", TRUE,
-			      "GtkContainer::border_width", 5, NULL);
-        if(is_save) {
-		gtk_window_set_title(GTK_WINDOW(window), _("Save game"));
-        	gtk_window_set_wmclass(GTK_WINDOW(window), "GtkBalls_Save", "GtkBalls");
-        } else {
-		gtk_window_set_title(GTK_WINDOW(window), _("Load game"));
-        	gtk_window_set_wmclass(GTK_WINDOW(window), "GtkBalls_Load", "GtkBalls");
-        }
 
-  	gtk_signal_connect(GTK_OBJECT(window), "delete_event", GTK_SIGNAL_FUNC(delete_event), NULL);
-  	gtk_signal_connect(GTK_OBJECT(window), "destroy", GTK_SIGNAL_FUNC(destroy), window);
-  	gtk_signal_connect(GTK_OBJECT(window), "key_press_event", GTK_SIGNAL_FUNC(key_pressed), NULL);
+	window=ut_window_new(is_save ? _("Save game") : _("Load game"),
+                             is_save ? "GtkBalls_Save" : "GtkBalls_Load",
+			     "GtkBalls", TRUE, TRUE, TRUE, 5);
 
   	vbox=gtk_vbox_new(FALSE, 0);
   	button_box=gtk_hbox_new(TRUE, 0);
-  	gtk_container_border_width(GTK_CONTAINER(vbox), 1);
+  	gtk_container_set_border_width(GTK_CONTAINER(vbox), 1);
 
         swindow=gtk_scrolled_window_new(NULL, NULL);
         gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(swindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-  	gtk_box_pack_start(GTK_BOX(vbox), swindow, FALSE, FALSE, 4);
-
-        clist=gtk_clist_new_with_titles(1, &titles);
-        gtk_clist_set_selection_mode(GTK_CLIST(clist), GTK_SELECTION_BROWSE);
-        GTK_WIDGET_UNSET_FLAGS(GTK_CLIST(clist)->column[0].button, GTK_CAN_FOCUS);
-
-	style = gtk_widget_get_style(clist);
+        gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(swindow), GTK_SHADOW_ETCHED_IN);
+  	gtk_box_pack_start(GTK_BOX(vbox), swindow, TRUE, TRUE, 0);
 
+        store=gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_STRING);
         for(i=0;i<num;i++) {
-                if((w = gdk_string_width(style->font, gamelist[i*2])) > width) {
-                        width = w;
-                }
-        	gtk_clist_append(GTK_CLIST(clist), &gamelist[i*2]);
-                g_free(gamelist[i*2]);
-                gtk_clist_set_row_data_full(GTK_CLIST(clist), i, g_strdup(gamelist[i*2+1]), save_load_game_list_destroy);
-                g_free(gamelist[i*2+1]);
+                gtk_list_store_append(store, &iter);
+                gtk_list_store_set(store, &iter, 0, (GValue *)gamelist[i*2], 1, (GValue *)gamelist[i*2+1], -1);
         }
-        gtk_clist_set_column_width(GTK_CLIST(clist), 0, width);
         g_free(gamelist);
 
-  	gtk_signal_connect(GTK_OBJECT(clist), "select_row", GTK_SIGNAL_FUNC(save_load_game_row_selected), NULL);
+      	treeview=gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
+	gtk_widget_set_size_request(treeview, -1, 250);
+
+      	g_object_unref(G_OBJECT(store));
+      	gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(treeview), TRUE);
+      	gtk_tree_view_set_search_column(GTK_TREE_VIEW(treeview), 0);
+        gtk_tree_selection_set_mode(gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview)), GTK_SELECTION_BROWSE);
+
+  	g_signal_connect(G_OBJECT(gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview))),
+		         "changed", G_CALLBACK(sl_row_activated), store);
 
         if(is_save) {
-        	gtk_clist_append(GTK_CLIST(clist), &empty);
-                gtk_clist_set_row_data(GTK_CLIST(clist), num, NULL);
-        	gtk_clist_select_row(GTK_CLIST(clist), num, 0);
-        } else {
-        	gtk_clist_select_row(GTK_CLIST(clist), 0, 0);
+                gtk_list_store_append(store, &iter);
+                gtk_list_store_set(store, &iter, 0, _("Empty"), -1);
         }
-  	gtk_container_add(GTK_CONTAINER(swindow), clist);
+  	gtk_container_add(GTK_CONTAINER(swindow), treeview);
 
+        renderer=gtk_cell_renderer_text_new();
+        column=gtk_tree_view_column_new_with_attributes(_("Date (score)"), renderer, "text", 0, NULL);
+        gtk_tree_view_column_set_sort_column_id(column, 0);
+        gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), column);
+
+        /* FIXME: imho this way of sorting is stupid */
+        gtk_tree_view_column_clicked(column);
+        gtk_tree_view_column_clicked(column);
+
+        if(iter.stamp==store->stamp) {
+                path=gtk_tree_path_new_from_string("0");
+                if(path) {
+			gtk_tree_selection_select_path(gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview)), path);
+                	gtk_tree_view_set_cursor(GTK_TREE_VIEW(treeview), path, NULL, FALSE);
+                	gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(treeview), path, NULL, TRUE, 0, 0);
+                	gtk_tree_path_free(path);
+                }
+        }
 
         if(is_save) {
-  		ok_button=gtk_button_new_with_label(_("Save game"));
-  		gtk_signal_connect(GTK_OBJECT(ok_button), "clicked", GTK_SIGNAL_FUNC(do_save_game), window);
+        	ok_button=ut_button_new(_("Save game"), do_save_game, window, button_box);
+  		g_signal_connect(G_OBJECT(treeview), "row_activated", G_CALLBACK(save_row_activated_cb), window);
         } else {
-  		ok_button=gtk_button_new_with_label(_("Load game"));
-  		gtk_signal_connect(GTK_OBJECT(ok_button), "clicked", GTK_SIGNAL_FUNC(do_load_game), window);
+        	ok_button=ut_button_new(_("Load game"), do_load_game, window, button_box);
+  		g_signal_connect(G_OBJECT(treeview), "row_activated", G_CALLBACK(load_row_activated_cb), window);
         }
-        GTK_WIDGET_SET_FLAGS(ok_button, GTK_CAN_DEFAULT | GTK_CAN_FOCUS);
-  	gtk_box_pack_start(GTK_BOX(button_box), ok_button, TRUE, TRUE, 0);
 
-  	cancel_button=gtk_button_new_with_label(_("Cancel"));
-        GTK_WIDGET_SET_FLAGS(cancel_button, GTK_CAN_DEFAULT | GTK_CAN_FOCUS);
-  	gtk_signal_connect(GTK_OBJECT(cancel_button), "clicked", GTK_SIGNAL_FUNC(destroy), window);
-  	gtk_box_pack_start(GTK_BOX(button_box), cancel_button, TRUE, TRUE, 0);
+        cancel_button=ut_button_new_stock_swap(GTK_STOCK_CANCEL, gtk_widget_destroy, window, button_box);
 
   	gtk_box_pack_start(GTK_BOX(vbox), button_box, FALSE, FALSE, 4);
 
   	gtk_container_add(GTK_CONTAINER(window), vbox);
 
-  	gtk_widget_show_all(window);
-  	gtk_grab_add(window);
         gtk_widget_grab_default(ok_button);
+        gtk_widget_grab_focus(ok_button);
+  	gtk_widget_show_all(window);
 }
 
 void save_game_cb(GtkWidget *widget, gpointer data) {
@@ -441,7 +456,7 @@ void init_names_scores_and_dates (GtkWidget **names, GtkWidget **scores, GtkWidg
   	gchar *str;
 
   	if(!read_score(Scoreboard, &FullScoreboard, &FullScoreboardNumber)) {
-      		simple_message_box(_("Unable to read score.\n"), _("OK"));
+      		simple_message_box(_("Unable to read score.\n"));
     	}
 
   	for (i=0;i<10;i++) {
@@ -470,22 +485,13 @@ void show_hall_of_fame(GtkWidget *widget, gpointer data) {
   	GtkWidget *close_button;
   	gint i;
 
-  	hall_of_fame=gtk_widget_new(gtk_window_get_type(),
-				    "GtkObject::user_data", NULL,
-				    "GtkWindow::title", _("Hall of Fame"),
-				    "GtkWindow::allow_grow", FALSE,
-				    "GtkWindow::allow_shrink", FALSE,
-				    "GtkContainer::border_width", 5, NULL);
-        gtk_window_set_wmclass(GTK_WINDOW(hall_of_fame), "GtkBalls_Scores", "GtkBalls");
-
-  	gtk_signal_connect(GTK_OBJECT(hall_of_fame), "delete_event", GTK_SIGNAL_FUNC(delete_event), NULL);
-  	gtk_signal_connect(GTK_OBJECT(hall_of_fame), "destroy", GTK_SIGNAL_FUNC(destroy), hall_of_fame);
-  	gtk_signal_connect(GTK_OBJECT(hall_of_fame), "key_press_event", GTK_SIGNAL_FUNC(key_pressed), NULL);
+	hall_of_fame=ut_window_new(_("Hall of Fame"), "GtkBalls_Scores", "GtkBalls", TRUE, TRUE, FALSE, 5);
 
   	frame=gtk_frame_new(_(" Hall of Fame "));
   	vbox=gtk_vbox_new(FALSE, 0);
+
   	button_box=gtk_hbox_new(TRUE, 0);
-  	gtk_container_border_width(GTK_CONTAINER(vbox), 1);
+  	gtk_container_set_border_width(GTK_CONTAINER(vbox), 1);
   	gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 0);
 
   	table=gtk_table_new(9, 3, FALSE);
@@ -499,42 +505,23 @@ void show_hall_of_fame(GtkWidget *widget, gpointer data) {
 
   	gtk_container_add(GTK_CONTAINER(frame), table);
 
-  	close_button=gtk_button_new_with_label(_("Close"));
-        GTK_WIDGET_SET_FLAGS(close_button, GTK_CAN_DEFAULT | GTK_CAN_FOCUS);
-  	gtk_signal_connect(GTK_OBJECT(close_button), "clicked", GTK_SIGNAL_FUNC(destroy), hall_of_fame);
-  	gtk_box_pack_start(GTK_BOX(button_box), close_button, TRUE, TRUE, 0);
+        close_button=ut_button_new_stock_swap(GTK_STOCK_CLOSE, gtk_widget_destroy, hall_of_fame, button_box);
 
   	gtk_box_pack_start(GTK_BOX(vbox), button_box, FALSE, FALSE, 4);
 
   	gtk_container_add(GTK_CONTAINER(hall_of_fame), vbox);
 
-  	gtk_widget_show_all(hall_of_fame);
-  	gtk_grab_add(hall_of_fame);
         gtk_widget_grab_default(close_button);
+  	gtk_widget_show_all(hall_of_fame);
 }
 
 /* shows simple message box */
-void simple_message_box(gchar *message, gchar *label) {
+void simple_message_box(gchar *message) {
   	GtkWidget *dialog;
-  	GtkWidget *prompt_label;
-  	GtkWidget *button;
 
-  	dialog=gtk_dialog_new();
-        gtk_window_set_wmclass(GTK_WINDOW(dialog), "GtkBalls_Dialog", "GtkBalls");
-  	gtk_container_border_width(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), 10);
-  	gtk_signal_connect(GTK_OBJECT(dialog), "key_press_event", GTK_SIGNAL_FUNC(key_pressed), NULL);
-
-  	prompt_label=gtk_label_new(message);
-  	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), prompt_label, TRUE, TRUE, 5);
-
-  	button=gtk_button_new_with_label(label);
-  	gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(destroy), dialog);
-  	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), button, TRUE, TRUE, 0);
-  	GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
-  	gtk_widget_grab_default(button);
-
-  	gtk_widget_show_all(dialog);
-  	gtk_grab_add(dialog);
+  	dialog=gtk_message_dialog_new(NULL, 0, GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE, message);
+ 	gtk_dialog_run(GTK_DIALOG (dialog));
+ 	gtk_widget_destroy(dialog);
 }
 
 /* returns board coordinate of the window position */
@@ -553,20 +540,15 @@ gint calculate_board_coord_y(gint window_coord) {
 }
 
 void update_rectangle(GtkWidget *widget, gint x, gint y, gint w, gint h) {
-  	GdkRectangle r;
-
-  	r.x = x;
-  	r.y = y;
-  	r.width = w;
-  	r.height = h;
-  	gtk_widget_draw(widget, &r);
+  	gdk_draw_drawable(widget->window, widget->style->fg_gc[GTK_WIDGET_STATE(widget)],
+			  Pixmap, x, y, x, y, w, h);
 }
 
 void set_hi_score_label(void) {
   	gchar *str;
 
   	str=g_strdup_printf(_("Hi-score: %i"), Hi_score);
-  	gtk_label_set(GTK_LABEL(Hi_score_label), str);
+  	gtk_label_set_text(GTK_LABEL(Hi_score_label), str);
         g_free(str);
 }
 
@@ -579,7 +561,7 @@ void set_user_score_label(void) {
     	}
 
   	str=g_strdup_printf(_("Your score: %i"), Score);
-  	gtk_label_set(GTK_LABEL(User_score_label), str);
+  	gtk_label_set_text(GTK_LABEL(User_score_label), str);
         g_free(str);
 }
 
@@ -812,7 +794,7 @@ gint destroy_lines(GtkWidget *widget, GdkPixmap *pixmap, gint count_score) {
                 }
 
       		draw_board(widget, pixmap);
-      		gtk_widget_draw(widget, NULL);
+      		update_rectangle(DrawingArea, 0, 0, -1, -1);
 
       		if(count_score) {
 	  		Score=Score+Rules.destroy*2+2*(number_of_balls_to_delete-Rules.destroy)*(number_of_balls_to_delete-Rules.destroy);
@@ -832,37 +814,46 @@ void read_entry(GtkWidget *widget, gpointer data) {
   	time_t current_time;
   	struct tm *timeptr;
   	gint pos;
+        gsize br, bw;
+        gchar *tstr;
   
-  	user_name=gtk_entry_get_text(GTK_ENTRY(data));
-  	if(!strlen(user_name)) {
-    		strcpy(current_entry.name, _("Anonymous"));
+  	user_name=(gchar*)gtk_entry_get_text(GTK_ENTRY(data));
+  	if(user_name && *user_name) {
+    		strncpy(current_entry.name, user_name, sizeof(current_entry.name));
   	} else {
-    		strcpy(current_entry.name, user_name);
+    		strncpy(current_entry.name, _("Anonymous"), sizeof(current_entry.name));
         }
-  	strcpy(last_player_name, user_name);
+  	strncpy(last_player_name, user_name, sizeof(last_player_name));
   	current_entry.score=saved_score;
   	current_time=time(NULL);
   	timeptr=localtime(&current_time);
   	if(!timeptr) {
-      		simple_message_box(_("Unable to determine current date.\n"), _("OK"));
-      		current_entry.date[0]='\0';
+      		simple_message_box(_("Unable to determine current date.\n"));
+                strncpy(current_entry.date, _("Unknown"), sizeof(current_entry.date));
     	} else {
       		/* I use _() for format string because some people might want
 	 	   format of the date more traditional for their locale */
-      		if(!strftime(current_entry.date, 30, _("%a %b %d %H:%M %Y"), timeptr)) {
-	  		simple_message_box(_("Unable to determine current date.\n"), _("OK"));
-	  		current_entry.date[0] = '\0';
-		}
+      		if(!strftime(current_entry.date, sizeof(current_entry.date), _("%a %b %d %H:%M %Y"), timeptr)) {
+	  		simple_message_box(_("Unable to determine current date.\n"));
+                        strncpy(current_entry.date, _("Unknown"), sizeof(current_entry.date));
+		} else {
+                	tstr=g_locale_to_utf8(current_entry.date, -1, &br, &bw, NULL);
+                        if(!tstr) {
+                                strncpy(current_entry.date, _("Unknown"), sizeof(current_entry.date));
+                        } else {
+                        	strncpy(current_entry.date, tstr, sizeof(current_entry.date));
+                                g_free(tstr);
+                        }
+                }
     	}
   
   	if(!read_score(Scoreboard, &FullScoreboard, &FullScoreboardNumber)) {
-      		simple_message_box (_("Unable to read score.\n"), _("OK"));
+      		simple_message_box (_("Unable to read score.\n"));
     	}
   	pos=insert_entry_in_score_board(Scoreboard, current_entry);
   	if(!write_score(Scoreboard, FullScoreboard, FullScoreboardNumber)) {
-      		simple_message_box(_("Unable to save score.\n"), _("OK"));
+      		simple_message_box(_("Unable to save score.\n"));
     	}
-  	gtk_grab_remove(dialog);
   	gtk_widget_destroy(dialog);
   	/* show scores to let user see if (s)he's on top ;) */
   	show_hall_of_fame(NULL, (gpointer)pos);
@@ -872,37 +863,40 @@ gchar *input_name_dialog(void) {
   	GtkWidget *prompt_label;
   	GtkWidget *name;
   	GtkWidget *button;
+        gchar *s;
 
   	/* we have to save score, because they will be set to 0 in new_game() */
   	saved_score=Score;
 
   	dialog=gtk_dialog_new();
         gtk_window_set_wmclass(GTK_WINDOW(dialog), "GtkBalls_Inputname", "GtkBalls");
-  	gtk_signal_connect(GTK_OBJECT(dialog), "key_press_event", GTK_SIGNAL_FUNC(key_pressed), NULL);
 
-  	gtk_container_border_width(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), 2);
+  	gtk_container_set_border_width(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), 2);
 
   	prompt_label=gtk_label_new(_("Enter your name"));
   	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), prompt_label, TRUE, TRUE, 0);
 
-  	name=gtk_entry_new_with_max_length(14);
-  	gtk_signal_connect(GTK_OBJECT(name), "activate", GTK_SIGNAL_FUNC(read_entry), name);
+  	name=gtk_entry_new();
+        gtk_entry_set_max_length(GTK_ENTRY(name), 14);
+  	g_signal_connect(G_OBJECT(name), "activate", G_CALLBACK(read_entry), name);
+
   	/* restore the last player's name */
-  	if(user_name) {
+        if(!(*last_player_name)) {
+                if((s = getenv("USER"))) {
+  			strncpy(last_player_name, s, sizeof(last_player_name));
+                }
+        }
+        if(*last_player_name) {
     		gtk_entry_set_text(GTK_ENTRY(name), last_player_name);
-    		gtk_entry_select_region(GTK_ENTRY(name), 0, -1);
-  	}
+    		gtk_editable_select_region(GTK_EDITABLE(name), 0, -1);
+        }
   	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), name, TRUE, TRUE, 5);
 
-  	button=gtk_button_new_with_label(_("OK"));
-  	gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(read_entry), name);
-  	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), button, TRUE, TRUE, 0);
+        button=ut_button_new_stock(GTK_STOCK_OK, read_entry, name, GTK_DIALOG(dialog)->action_area);
 
         gtk_widget_grab_focus(name);
 
-  	GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
   	gtk_widget_grab_default(button);
-
   	gtk_widget_show_all(dialog);
 
   	return NULL;
@@ -922,15 +916,10 @@ void rules_ok(GtkWidget *widget, gpointer data) {
   	if(data) {
       		gtk_widget_destroy(GTK_WIDGET(data));
   	}
+        save_preferences();
 }
 
-void rules_cancel(GtkWidget *widget, gpointer data) {
-  	if(data) {
-      		gtk_widget_destroy(GTK_WIDGET(data));
-  	}
-}
-
-void rules_classic(GtkWidget *widget, gpointer data) {
+void rules_classic(void) {
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(RD.xrange), ClassicRules.xsize);
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(RD.yrange), ClassicRules.ysize);
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(RD.crange), ClassicRules.colors);
@@ -945,11 +934,7 @@ void rules_dialog(void) {
   	GtkWidget *separator;
   	GtkWidget *ok_button, *cancel_button, *classic_button;
 
-  	dialog=gtk_window_new(GTK_WINDOW_TOPLEVEL);
-  	gtk_window_set_title(GTK_WINDOW(dialog), _("Game rules"));
-        gtk_window_set_wmclass(GTK_WINDOW(dialog), "GtkBalls_Rules", "GtkBalls");
-  	gtk_signal_connect(GTK_OBJECT(dialog), "key_press_event", GTK_SIGNAL_FUNC(key_pressed), NULL);
-  	gtk_container_border_width(GTK_CONTAINER(dialog), 5);
+	dialog=ut_window_new(_("Game rules"), "GtkBalls_Rules", "GtkBalls", TRUE, TRUE, TRUE, 5);
 
   	big_vbox=gtk_vbox_new(FALSE, 0);
   	gtk_container_add(GTK_CONTAINER(dialog), big_vbox);
@@ -966,7 +951,7 @@ void rules_dialog(void) {
 	RD.nrange=ut_spin_button_new(_("Number of 'next' objects"), 2, 99, Rules.next, vbox);
 	RD.drange=ut_spin_button_new(_("How many balls at line eliminate it"), 3, 99, Rules.destroy, vbox);
 
-	classic_button=ut_button_new(_("Classic rules"), rules_classic, GTK_OBJECT(dialog), vbox);
+	classic_button=ut_button_new(_("Classic rules"), rules_classic, dialog, vbox);
 
   	separator=gtk_hseparator_new();
   	gtk_box_pack_start(GTK_BOX(big_vbox), separator, FALSE, FALSE, 5);
@@ -975,10 +960,11 @@ void rules_dialog(void) {
         gtk_button_box_set_layout(GTK_BUTTON_BOX(buttons_box), GTK_BUTTONBOX_SPREAD);
   	gtk_box_pack_start(GTK_BOX(big_vbox), buttons_box, TRUE, TRUE, 0);
 
-	ok_button=ut_button_new(_("OK"), GTK_SIGNAL_FUNC(rules_ok), GTK_OBJECT(dialog), buttons_box);
-	cancel_button=ut_button_new(_("Cancel"), GTK_SIGNAL_FUNC(rules_cancel), GTK_OBJECT(dialog), buttons_box);
+        ok_button=ut_button_new_stock(GTK_STOCK_OK, rules_ok, dialog, buttons_box);
+
+        cancel_button=ut_button_new_stock_swap(GTK_STOCK_CANCEL, gtk_widget_destroy, dialog, buttons_box);
+
+        gtk_widget_grab_default(ok_button);
 
   	gtk_widget_show_all(dialog);
-        gtk_widget_grab_default(ok_button);
-  	gtk_grab_add(dialog);
 }
